from typing import List

from beam.base import AbstractDataLoader
from beam.serializer import FileConfiguration
from beam.types import OutputType


class OutputManager(AbstractDataLoader):
    def __init__(self) -> None:
        self.files: List[FileConfiguration] = []
        self.dirs: List[FileConfiguration] = []

    def File(self, path: str, name: str, **_):
        self.files.append(
            FileConfiguration(path=path, name=name, output_type=OutputType.File)
        )

    def Dir(self, path: str, name: str, **_):
        self.dirs.append(
            FileConfiguration(path=path, name=name, output_type=OutputType.Directory)
        )

    def dumps(self):
        return [
            *[f.validate_and_dump() for f in self.files],
            *[d.validate_and_dump() for d in self.dirs],
        ]

    def from_config(self, outputs: List[dict]):
        if outputs is None:
            return

        for f in outputs:
            if f.get("output_type") == OutputType.Directory:
                self.Dir(**f)
            else:
                self.File(**f)



import os
import json
import inspect

from typing import List, Union
from beam.base import AbstractDataLoader
from beam.configs.outputs import OutputManager
from beam.configs.trigger import TriggerManager
from beam.configs.mount import MountManager
from beam.serializer import AppSpecConfiguration
from beam.types import PythonVersion
from beam.utils.parse import compose_cpu, compose_memory, load_requirements_file


class App(AbstractDataLoader):
    def __init__(
        self,
        *,
        name: str,
        cpu: Union[str, int],
        memory: str,
        gpu: int = 0,
        apt_install: List[str] = [],
        python_version: PythonVersion = PythonVersion.Python38,
        python_packages: Union[str, List[str]] = [],
        workspace: str = "./"
    ) -> None:
        """
        Keyword Arguments:
            name: the unique identifier for your app
            cpu: total total cpu cores available to app
            memory: total amount of memory available to app
                - in format [Number][Mi|Gi]
                - e.g. 12Gi or 250Mi
            (Optional) gpu: total gpu devices available to app
            (Optional) apt_install: system level dependencies you want installed into the app
                - e.g. libssl-dev
            (Optional) python_version: version of python to run your app code
            (Optional) python_packages: list of python packages you want to install, or the path to a requirements.txt file
                - e.g. "torch" or "torch==1.12.0"
            (Optional) workspace: directory to continously sync to app during development
        """

        if isinstance(python_packages, str):
            python_packages = load_requirements_file(python_packages)

        self.Spec = AppSpecConfiguration(
            name=name,
            cpu=compose_cpu(cpu),
            gpu=gpu,
            memory=compose_memory(memory),
            apt_install=apt_install,
            python_version=python_version,
            python_packages=python_packages,
            workspace=workspace,
        )

        self.Trigger = TriggerManager()
        self.Output = OutputManager()
        self.Mount = MountManager()

    def dumps(self):
        return json.dumps(
            {
                "app": self.Spec.validate_and_dump(),
                "triggers": self.Trigger.dumps(),
                "outputs": self.Output.dumps(),
                "mounts": self.Mount.dumps(),
                "autoscaling": self.Trigger.AutoScaling.dumps(),
            }
        )

    @staticmethod
    def from_config(config: Union[dict, str]) -> "App":
        if isinstance(config, str):
            config = json.loads(config)

        app_config = config.get("app")
        triggers = config.get("triggers")
        outputs = config.get("outputs")
        mounts = config.get("mounts")
        autoscaling = config.get("autoscaling")

        app = App(**app_config)
        app.Trigger.from_config(triggers)
        app.Output.from_config(outputs)
        app.Mount.from_config(mounts)
        app.Trigger.AutoScaling.from_config(autoscaling)

        return app



import importlib.util
import inspect
import sys

from beam import App


def build(path: str) -> str:
    beamapp = None
    spec = importlib.util.spec_from_file_location("app_module", path)
    app_module = importlib.util.module_from_spec(spec)
    sys.modules["app_module"] = app_module
    spec.loader.exec_module(app_module)

    for member in inspect.getmembers(app_module):
        member_value = member[1]
        if isinstance(member_value, App):
            beamapp = member_value
            break

    if beamapp is not None:
        return beamapp.dumps()

    raise Exception("Beam app not found")


if __name__ == "__main__":
    if len(sys.argv) < 2:
        exit(1)

    path = sys.argv[1]
    save_stdout = sys.stdout
    sys.stdout = None
    app_config = build(path)
    sys.stdout = save_stdout
    print(app_config)



import os
import argparse
import pathlib
import shutil
import jinja2

from .build import build

templates_path = os.path.join(
    pathlib.Path(__file__).parent.parent.resolve(), "templates"
)


def create_template(name: str, **kwargs):
    environment = jinja2.Environment(loader=jinja2.FileSystemLoader(templates_path))
    template = environment.get_template("base.jinja")
    rendered_template = template.render(name=name, **kwargs)

    app_path = os.path.join(os.getcwd(), "app.py")

    if os.path.exists(app_path):
        app_path = os.path.join(os.getcwd(), f"app-{name}.py")

    with open(app_path, "w") as f:
        f.write(rendered_template)

    try:
        os.environ["SKIP_VALIDATION"] = "1"
        build(app_path)
    except BaseException as e:
        os.remove(app_path)
        raise e


def parse_args():
    parser = argparse.ArgumentParser(description="Create a new app")
    parser.add_argument("--name", type=str, help="Name of the app")
    parser.add_argument("--cpu", type=int, help="CPU", default=1)
    parser.add_argument("--memory", type=str, help="Memory", default="2Gi")
    parser.add_argument("--trigger", type=str, help="Trigger")

    return parser.parse_args()


if __name__ == "__main__":
    parsed_args = parse_args()
    create_template(**vars(parsed_args))

    if parsed_args.trigger:
        shutil.copyfile(
            os.path.join(templates_path, "run.py"), os.path.join(os.getcwd(), "run.py")
        )



import json
import beam
import pytest

from beam.types import PythonVersion
from beam.utils.parse import compose_cpu, compose_memory
from beam.exceptions import BeamSerializationError


def valid_reconstruction(app: beam.App):
    dumped_app = app.dumps()
    dumped_json = json.dumps(dumped_app)
    app_from_dumped_config = beam.App.from_config(json.loads(dumped_json))
    return app_from_dumped_config.dumps() == dumped_app


def test_basic_app_reconstruction():
    app = beam.App(
        name="some_app",
        cpu="4000m",
        memory="128mi",
        gpu=1,
        python_packages=["pytorch"],
        python_version=PythonVersion.Python37,
        workspace="./some_folder",
    )

    assert valid_reconstruction(app)


REQUIREMENTS_FILE_CONTENT = """

torch
numpy==1.19.5

scikit-learn

"""


def test_python_packages_from_requirements_file(tmp_path):
    requirements_path = tmp_path / "requirements.txt"
    requirements_path.write_text(REQUIREMENTS_FILE_CONTENT)
    assert requirements_path.read_text() == REQUIREMENTS_FILE_CONTENT

    app = beam.App(
        name="some_app",
        cpu="4000m",
        memory="128mi",
        gpu=1,
        python_packages=str(requirements_path),
        python_version=PythonVersion.Python37,
        workspace="./some_folder",
    )

    assert len(app.Spec.python_packages) == 3
    assert ["torch", "numpy==1.19.5", "scikit-learn"] == app.Spec.python_packages

    assert valid_reconstruction(app)


def test_valid_schedule_expressions():
    app1 = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=1,
    )

    app1.Trigger.Schedule(
        when="* * * * *",
        handler="method.py:run",
    )

    assert valid_reconstruction(app1)

    app1 = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=1,
    )

    app1.Trigger.Schedule(
        when="every 5m",
        handler="method.py:run",
    )

    assert valid_reconstruction(app1)

    app2 = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=1,
    )

    app2.Trigger.Schedule(
        when="every",
        handler="method.py:run",
    )

    with pytest.raises(BeamSerializationError):
        valid_reconstruction(app2)


def test_multiple_triggers_should_fail_build():
    app = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=1,
    )

    app.Trigger.Schedule(
        when="* * * * *",
        handler="test.py:app",
    )

    with pytest.raises(ValueError):
        app.Trigger.Webhook(
            inputs={
                "some_input": beam.Types.String(),
            },
            handler="test.py:app",
        )


def test_compose_cpu():
    assert compose_cpu(1) == "1000m"
    assert compose_cpu(1.999) == "1999m"
    assert compose_cpu("1000m") == "1000m"


def test_compose_memory():
    assert compose_memory("10gi") == "10Gi"
    assert compose_memory("10 gi ") == "10Gi"
    assert compose_memory("10 Gi ") == "10Gi"

    # Raises if Gi is > 256
    with pytest.raises(ValueError):
        compose_memory("10000Gi")

    assert compose_memory("256mi") == "256Mi"
    assert compose_memory("256 mi ") == "256Mi"
    assert compose_memory("256 Mi ") == "256Mi"

    # Raises if Mi is < 128 or >= 1000
    with pytest.raises(ValueError):
        compose_memory("127Mi")

    with pytest.raises(ValueError):
        compose_memory("1000Mi")

    # Test invalid formats
    with pytest.raises(ValueError):
        compose_memory("1000ti")

    with pytest.raises(ValueError):
        compose_memory("1000.0")

    with pytest.raises(ValueError):
        compose_memory("1000")

    with pytest.raises(ValueError):
        compose_memory("2 gigabytes")


def test_cpu_in_app_config():
    config = {"name": "some_app", "memory": "128mi"}

    app1 = beam.App(
        **config,
        cpu=4,
    )

    app2 = beam.App(
        **config,
        cpu="4000m",
    )

    assert valid_reconstruction(app1)
    assert valid_reconstruction(app2)
    assert app1.dumps() == app2.dumps()

    app3 = beam.App(
        **config,
        cpu=1,
    )

    app4 = beam.App(**config, cpu="1000m")

    assert valid_reconstruction(app3)
    assert valid_reconstruction(app4)
    assert app3.dumps() == app4.dumps()



import dataclasses
import os

from abc import abstractmethod
from copy import deepcopy
from marshmallow_dataclass import dataclass
from typing import ClassVar, Type as TypingType
from marshmallow import Schema
from beam.exceptions import BeamSerializationError
from beam.types import Types


class AbstractDataLoader:
    @abstractmethod
    def dumps(self):
        pass

    @abstractmethod
    def from_config(self):
        pass


@dataclass
class BaseSerializer:
    Schema: ClassVar[TypingType[Schema]] = Schema

    def validate_and_dump(self):
        validation_errors = self.validate()

        if len(validation_errors) > 0:
            raise BeamSerializationError(
                f"{self.__class__.__name__}\n{validation_errors}"
            )
        return self.dump()

    def validate(self):
        if os.getenv("SKIP_VALIDATION") is not None:
            return []

        data_to_val = {}

        for key, val in self.__dataclass_fields__.items():
            if isinstance(val, dataclasses.Field) and key != "Schema":
                data_to_val[key] = self.__getattribute__(key)

        validation_errors = []
        validation_errors = self.Schema().validate(data_to_val)

        if len(validation_errors) > 0:
            raise BeamSerializationError(
                f"{self.__class__.__name__}\n{validation_errors}"
            )

        return validation_errors

    def dump(self, data=None):
        if data:
            return self.Schema().dump(data)

        return self.Schema().dump(self)

    class Meta:
        ordered = True


class BaseTriggerSerializer(BaseSerializer):
    def dump(self):
        config = deepcopy(self)

        if hasattr(config, "inputs"):
            config.inputs = Types.dump_schema(config.inputs)

        if hasattr(config, "outputs"):
            config.outputs = Types.dump_schema(config.outputs)

        return super().dump(data=config)



import base64
import builtins
import copy
import io
import json
import os

from abc import ABC, abstractmethod
from importlib import import_module


class InvalidTypeException(Exception):
    def __init__(self, msg, errors=None):
        super().__init__(msg)
        self.msg = msg
        self.errors = errors


class InvalidPayloadException(Exception):
    def __init__(self, msg, errors=None):
        super().__init__(msg)
        self.msg = msg
        self.errors = errors


class TypeSerializer(ABC):
    def __init__(self, *args, **kwargs):
        return self.define(*args, **kwargs)

    @abstractmethod
    def define(self, *args, **kwargs):
        pass

    @abstractmethod
    def schema(self):
        pass

    @abstractmethod
    def serialize(self, obj):
        pass

    @abstractmethod
    def deserialize(self, data):
        pass


class FloatType(TypeSerializer):
    def define(self):
        pass

    def schema(self):
        return {"type": "float"}

    def serialize(self, obj):
        try:
            output = float(obj)
        except ValueError:
            raise InvalidTypeException("invalid_float_value")

        return output

    def deserialize(self, data):
        try:
            obj = float(data)
        except ValueError:
            raise InvalidTypeException("invalid_float_value")

        return obj


class StringType(TypeSerializer):
    def define(self, max_length=None):
        self.max_length = max_length

    def schema(self):
        return {"type": "string", "max_length": self.max_length}

    def serialize(self, obj):
        output = str(obj)

        if self.max_length:
            if len(output) > self.max_length:
                raise InvalidTypeException("invalid_string_too_long")

        return output

    def deserialize(self, data):
        obj = str(data)

        if self.max_length:
            if len(obj) > self.max_length:
                raise InvalidTypeException("invalid_string_too_long")

        return obj


class JsonType(TypeSerializer):
    def define(self):
        pass

    def schema(self):
        return {"type": "json"}

    def serialize(self, obj):
        if not isinstance(obj, dict) and not isinstance(obj, list):
            raise InvalidTypeException("invalid_json_type")

        try:
            output = json.loads(json.dumps(obj))
        except TypeError:
            raise InvalidTypeException("invalid_json")

        return output

    def deserialize(self, data):
        if isinstance(data, str):
            try:
                obj = json.loads(data)
            except ValueError:
                raise InvalidTypeException("invalid_json_str")
        elif isinstance(data, dict) or isinstance(data, list):
            obj = data
        else:
            raise InvalidTypeException("invalid_json")

        return obj


class ImageType(TypeSerializer):
    def define(self, size=None, mode=None, raw=False):
        self.size = size
        self.mode = mode
        self.raw = raw

    def schema(self):
        return {
            "type": "image",
            "size": list(self.size) if self.size is not None else None,
            "mode": str(self.mode) if self.mode is not None else None,
            "raw": self.raw,
        }

    def serialize(self, obj):
        try:
            PIL_Image = import_module("PIL.Image")
        except ModuleNotFoundError:
            raise RuntimeError("pillow_required_for_image_inputs")

        # Load image from disk
        if isinstance(obj, str) and os.path.isfile(obj):
            obj = PIL_Image.open(obj)

        # Load image from base64
        elif isinstance(obj, str):
            base64_data = obj

            if "," in base64_data:
                base64_data = obj.split(",")[1]

            obj = PIL_Image.open(io.BytesIO(base64.b64decode(base64_data)))

        if self.size is not None and obj.size != self.size:
            raise InvalidTypeException("invalid_image_size")

        if self.mode is not None and obj.mode != self.mode:
            raise InvalidTypeException("invalid_image_mode")

        buffer = io.BytesIO()
        obj.save(buffer, format="PNG")
        output = base64.b64encode(buffer.getvalue()).decode("ascii")
        return output

    def deserialize(self, data):
        if self.raw:
            return str(data)

        try:
            PIL_Image = import_module("PIL.Image")
        except ModuleNotFoundError:
            raise RuntimeError("pillow_required_for_image_inputs")

        if "," in data:
            data = data.split(",")[1]

        obj = PIL_Image.open(io.BytesIO(base64.b64decode(data)))

        if self.size is not None and obj.size != self.size:
            raise InvalidTypeException("invalid_image_size")

        if self.mode is not None and obj.mode != self.mode:
            raise InvalidTypeException("invalid_image_mode")

        return obj


class BinaryType(TypeSerializer):
    def define(self):
        pass

    def schema(self):
        return {"type": "binary"}

    def serialize(self, obj):
        # Load binary from disk
        if isinstance(obj, str) and os.path.isfile(obj):
            with open(obj, "rb") as f:
                obj = f.read()

        try:
            output = base64.b64encode(obj).decode("ascii")
        except TypeError:
            raise InvalidTypeException("invalid_binary_data")

        return output

    def deserialize(self, data):
        try:
            return io.BytesIO(base64.b64decode(data)).getvalue()
        except TypeError:
            raise InvalidTypeException("invalid_binary_data")


class NumpyType(TypeSerializer):
    def define(self, shape=None, dtype=float):
        self.shape = shape
        if isinstance(dtype, type):
            self.dtype = dtype
        elif isinstance(dtype, str):
            self.dtype = getattr(builtins, dtype)
        else:
            raise InvalidTypeException("invalid_numpy_type")

    def schema(self):
        return {
            "type": "numpy",
            "shape": list(self.shape) if self.shape is not None else None,
            "dtype": self.dtype.__name__ if self.dtype is not None else None,
        }

    def serialize(self, obj):
        if self.shape is not None and obj.shape != self.shape:
            raise InvalidTypeException("invalid_numpy_shape")

        if self.dtype is not None:
            obj = obj.astype(self.dtype)

        return obj.tolist()

    def deserialize(self, data):
        try:
            np = import_module("numpy")
        except ModuleNotFoundError:
            raise RuntimeError("numpy_required_for_numpy_inputs")

        obj = np.array(data, dtype=self.dtype)

        errors = []
        if self.shape is not None and obj.shape != self.shape:
            errors = ["invalid_shape"]

        if errors:
            raise InvalidTypeException("invalid_numpy_shape", errors=errors)

        return obj


class DataframeType(TypeSerializer):
    def define(self, max_rows=None, max_cols=None):
        self.max_rows = max_rows
        self.max_cols = max_cols

    def schema(self):
        return {
            "type": "dataframe",
            "max_rows": self.max_rows,
            "max_cols": self.max_cols,
        }

    def serialize(self, obj):
        return obj.to_dict()

    def deserialize(self, data):
        try:
            pd = import_module("pandas")
        except ModuleNotFoundError:
            raise RuntimeError("pandas_required_for_dataframe_inputs")

        if isinstance(data, str):
            try:
                obj = pd.read_json(data)
            except ValueError:
                raise InvalidTypeException("invalid_dataframe_str")
        elif isinstance(data, dict):
            try:
                obj = pd.DataFrame.from_dict(data)
            except ValueError:
                raise InvalidTypeException("invalid_dataframe_dict")
        elif isinstance(data, pd.DataFrame):
            return data
        else:
            raise InvalidTypeException("invalid_dataframe")

        return obj


class TensorType(TypeSerializer):
    def define(self, shape=None, dtype=float):
        self.shape = shape
        self.dtype = dtype

    def schema(self):
        return {
            "type": "tensor",
            "shape": list(self.shape) if self.shape is not None else None,
            "dtype": self.dtype.__name__ if self.dtype is not None else None,
        }

    def serialize(self, obj):
        if self.shape is not None and obj.shape != self.shape:
            raise InvalidTypeException("invalid_tensor_shape")

        if self.dtype is not None:
            obj = obj.to(self.dtype)

        return obj.tolist()

    def deserialize(self, data):
        try:
            torch = import_module("torch")
        except ModuleNotFoundError:
            raise RuntimeError("torch_required_for_tensor_inputs")

        obj = torch.tensor(data)
        obj = obj.to(self.dtype)

        if self.shape is not None and obj.shape != self.shape:
            raise InvalidTypeException("invalid_tensor_shape")

        return obj


class BooleanType(TypeSerializer):
    """
    In the case of types where both the serialized and deserialized values are
    valid in both JSON payloads, as well as python objects, we can re-use the same
    logic for serialize/deserialize. For example, you can pass a boolean object
    as JSON, but you can also use that type directly in application logic as a boolean.
    """

    def define(self, **_):
        pass

    def schema(self):
        return {"type": "boolean"}

    def _to_boolean(self, obj):
        if isinstance(obj, bool):
            return obj

        elif isinstance(obj, str):
            if obj.lower() == "true":
                return True
            elif obj.lower() == "false":
                return False
            else:
                raise InvalidTypeException("invalid_boolean_value")

        elif isinstance(obj, int) or isinstance(obj, float):
            obj = int(obj)
            if obj == 1:
                return True
            elif obj == 0:
                return False
            else:
                raise InvalidTypeException("invalid_boolean_value")

        return obj

    def serialize(self, obj):
        return self._to_boolean(obj)

    def deserialize(self, data):
        return self._to_boolean(data)


class PythonVersion:
    Python37 = "python3.7"
    Python38 = "python3.8"
    Python39 = "python3.9"
    Python310 = "python3.10"

    Types = (
        (Python37, "python3.7"),
        (Python38, "python3.8"),
        (Python39, "python3.9"),
        (Python310, "python3.10"),
    )


class OutputType:
    Directory = "directory"
    File = "file"

    Types = ((Directory, "directory"), (File, "file"))


class MountType:
    Persistent = "persistent"

    Types = ((Persistent, "persistent"),)


class AutoscalingType:
    MaxRequestLatency = "max_request_latency"

    Types = ((MaxRequestLatency, "max_request_latency"),)


class Types:
    Float = FloatType
    String = StringType
    Json = JsonType
    NumpyArray = NumpyType
    Dataframe = DataframeType
    Tensor = TensorType
    Image = ImageType
    Binary = BinaryType
    Boolean = BooleanType

    type_registry = {
        "float": FloatType,
        "string": StringType,
        "json": JsonType,
        "numpy": NumpyType,
        "dataframe": DataframeType,
        "tensor": TensorType,
        "image": ImageType,
        "binary": BinaryType,
        "boolean": BooleanType,
    }

    @staticmethod
    def serialize(objects, schema):
        serialized_data = {}
        errors = []

        if not isinstance(objects, dict):
            raise InvalidTypeException("invalid_object_format")

        for key in schema.keys():
            if key not in objects.keys() and getattr(schema[key], "required", True):
                errors.append(f"{key}:required")
            elif (
                key not in objects.keys()
                and getattr(schema[key], "required", True) is False  # noqa
            ):
                continue
            else:
                _type = schema[key]
                _input = objects[key]

                try:
                    serialized_data[key] = _type.serialize(_input)
                except InvalidTypeException as exc:
                    errors.append(f"{key}:{exc}")

        if errors:
            raise InvalidPayloadException("invalid_object", errors=errors)

        return serialized_data

    @staticmethod
    def deserialize(data, schema):
        deserialized_payload = {}
        errors = []

        if not isinstance(data, dict):
            raise InvalidTypeException("invalid_data_format")

        for key in schema.keys():
            if key not in data.keys() and getattr(schema[key], "required", True):
                errors.append(f"{key}:required")
            elif (
                key not in data.keys()
                and getattr(schema[key], "required", True) is False  # noqa
            ):
                continue
            else:
                _type = schema[key]
                _input = data[key]

                try:
                    deserialized_payload[key] = _type.deserialize(_input)
                except InvalidTypeException as exc:
                    errors.append(f"{key}:{exc}")

        if errors:
            raise InvalidPayloadException("invalid_data", errors=errors)

        return deserialized_payload

    @staticmethod
    def dump_schema(schema):
        dumped = {}
        for key, val in schema.items():
            dumped[key] = val.schema()

            if getattr(val, "required", True) is False:
                dumped[key]["required"] = False

        return dumped

    @staticmethod
    def load_schema(description):
        if not isinstance(description, dict):
            raise InvalidTypeException("invalid_schema_description")

        schema = {}
        for key, val in description.items():
            if "type" not in val or val["type"] not in Types.type_registry:
                raise InvalidTypeException("invalid_schema_field_type")

            schema[key] = Types.type_registry[val["type"]]()
            kwargs = copy.copy(val)
            del kwargs["type"]
            schema[key].define(**kwargs)

            if kwargs.get("required", True) is False:
                setattr(schema[key], "required", False)

        return schema

    @staticmethod
    def to_list():
        t_list = []
        type_dict = Types.type_registry

        for key in type_dict:
            t_list.append(key)

        return t_list



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['beam',
 'beam.configs',
 'beam.scripts',
 'beam.templates',
 'beam.tests',
 'beam.utils']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'croniter>=1.3.7,<2.0.0',
 'marshmallow-dataclass>=8.5.9,<9.0.0',
 'marshmallow==3.18.0',
 'mypy>=0.981,<0.982',
 'typeguard>=2.13.3,<3.0.0']

setup_kwargs = {
    'name': 'beam-sdk',
    'version': '0.9.20',
    'description': '',
    'long_description': 'None',
    'author': 'luke lombardi',
    'author_email': 'luke@slai.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Dict, List, Optional, Union

from beam.base import AbstractDataLoader
from beam.serializer import (
    RestAPITrigger,
    CronJobTrigger,
    WebhookTrigger,
)
from beam.configs.autoscaling import AutoscalingManager
from beam.types import Types


class TriggerType:
    Webhook = "webhook"
    RestAPI = "rest_api"
    CronJob = "cron_job"


class TriggerManager(AbstractDataLoader):
    def __init__(self) -> None:
        self.webhooks: List[
            WebhookTrigger,
        ] = []
        self.cron_jobs: List[CronJobTrigger] = []
        self.rest_apis: List[RestAPITrigger] = []
        self.AutoScaling: AutoscalingManager = AutoscalingManager()

    def _validate_trigger_groupings(self):
        """
        NOTE: For the time being, the Beam APP can only accept one trigger during the alpha
        stages. Later we will allow multiple trigger types for webhooks (Slack, Twitter, etc)
        """
        triggers = self.webhooks + self.cron_jobs + self.rest_apis

        if len(triggers) > 1:
            raise ValueError("App can only have 1 trigger at a time")

    def Webhook(
        self,
        inputs: Dict[str, Types],
        handler: str,
        max_pending_tasks: Optional[int] = 1000,
        **_,
    ):
        """
        Arguments:
            handler: specify what method in which file to use as the entry point
            inputs: dictionary specifying how to serialize/deserialize input arguments
            max_pending_tasks (optional): maximum number of pending tasks in the queue, if exceeded, the webhook will prevent further items from being enqueued

        """
        self.webhooks.append(
            WebhookTrigger(
                inputs=inputs, handler=handler, max_pending_tasks=max_pending_tasks
            )
        )
        self._validate_trigger_groupings()

    def Schedule(self, when: str, handler: str, **_):
        """
        Arguments:
            handler: specify what method in which file to use as the entry point
            when: CRON string to indicate the schedule in which the job is to run
                - https://en.wikipedia.org/wiki/Cron \
        """
        self.cron_jobs.append(
            CronJobTrigger(when=when, handler=handler),
        )
        self._validate_trigger_groupings()

    def RestAPI(
        self,
        inputs: Dict[str, Types],
        outputs: Dict[str, Types],
        handler: str,
        loader: Optional[str] = None,
        keep_warm_seconds: Optional[int] = None,
        **_,
    ):
        """
        Arguments:
            handler: specify what method in which file to use as the entry point
            loader: specify the method that will load the data into the handler (optional)
            inputs: dictionary specifying how to serialize/deserialize input arguments
            outputs: dictionary specifying how to serialize/deserialize return values
            keep_warm_seconds: how long the app should stay warm after the last request
        """
        self.rest_apis.append(
            RestAPITrigger(
                inputs=inputs,
                outputs=outputs,
                handler=handler,
                loader=loader,
                keep_warm_seconds=keep_warm_seconds,
            )
        )
        self._validate_trigger_groupings()

    def get_trigger(self) -> Union[RestAPITrigger, CronJobTrigger, WebhookTrigger]:
        self._validate_trigger_groupings()
        triggers = self.webhooks + self.cron_jobs + self.rest_apis

        if len(triggers) == 0:
            return None

        return triggers[0]

    def dumps(self):
        # To make this backwards compatible in the future after switching back to
        # multiple triggers, we will make this a list that currently will only have 1 trigger
        self._validate_trigger_groupings()
        triggers = []

        if len(self.webhooks) != 0:
            triggers.append(self.webhooks[0].validate_and_dump())
        elif len(self.cron_jobs) != 0:
            triggers.append(self.cron_jobs[0].validate_and_dump())
        elif len(self.rest_apis) != 0:
            triggers.append(self.rest_apis[0].validate_and_dump())

        return triggers

    def from_config(self, triggers):
        if triggers is None:
            return

        for t in triggers:
            trigger_type = t.get("trigger_type")
            inputs = {}
            outputs = {}

            if t.get("inputs") is not None:
                inputs = Types.load_schema(t.get("inputs"))
                del t["inputs"]

            if t.get("outputs") is not None:
                outputs = Types.load_schema(t.get("outputs"))
                del t["outputs"]

            if trigger_type == TriggerType.Webhook:
                self.Webhook(**t, inputs=inputs)
            elif trigger_type == TriggerType.CronJob:
                self.Schedule(**t)
            elif trigger_type == TriggerType.RestAPI:
                self.RestAPI(**t, inputs=inputs, outputs=outputs)
            else:
                raise ValueError(
                    f"Found an unknown trigger type in config: {trigger_type}"
                )
